package be.iminds.ilabt.jfed.highlevel.jobs.report;

import be.iminds.ilabt.jfed.call_log_output.LogOutput;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.jobs.AbstractJob;
import be.iminds.ilabt.jfed.highlevel.util.LogEntryListener;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import javax.annotation.Nonnull;

/**
 * User: twalcari
 * Date: 12/9/2015
 * Time: 16:02
 */
public class JobReport implements LogEntryListener {
    private final AbstractJob<?> job;

    private final ObservableList<JobStateReport> jobStateReports = FXCollections.observableArrayList();
    private JobStateReport currentStateReport = null;

    public JobReport(@Nonnull AbstractJob<?> job) {
        this.job = job;

        assert job.getState() == null : "Cannot attach report to active job";

        job.stateProperty().addListener((observable, oldValue, newValue) -> {
            final JobStateReport jobStateReport = new JobStateReport(newValue);
            newValue.registerStateFinishedListener(state -> jobStateReport.registerEnd(state.getStatus()));

            jobStateReports.add(jobStateReport);
            currentStateReport = jobStateReport;

        });


        job.addTaskExecutionFinishedListener(this::onTaskExecutionFinished);
    }

    private void onTaskExecutionFinished(TaskExecution<?> taskExecution) {
        assert currentStateReport != null;
        currentStateReport.addTaskExecution(taskExecution);
    }

    public ObservableList<JobStateReport> getJobStateReports() {
        return jobStateReports;
    }

    public void log(@Nonnull LogOutput.LogEntry logEntry) {
        if (currentStateReport != null)
            currentStateReport.addLogLine(logEntry);
    }

    public String getName(){
        return job.getName();
    }


}
